package org.greenstone.gatherer.gui;
/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.Message;
import org.greenstone.gatherer.WGet;
import org.greenstone.gatherer.file.FileSystemModel;
import org.greenstone.gatherer.gui.Filter;
import org.greenstone.gatherer.gui.GComboBox;
import org.greenstone.gatherer.gui.tree.DragTree;
import org.greenstone.gatherer.util.DragGroup;
import org.greenstone.gatherer.util.GURL;
import org.greenstone.gatherer.util.TreeSynchronizer;
import org.greenstone.gatherer.util.Utility;
/**
 * @author John Thompson, Greenstone Digital Library, University of Waikato
 * @version 2.1
 */
public class MirrorPane
	 extends JPanel
	 implements ActionListener {

	 private ButtonGroup destination_folder_options;
	 private DragTree workspace_tree = null;
	 private FileSystemModel model = null;
	 /** The filter currently applied to the workspace tree. */
	 private Filter workspace_filter = null;

	 private ImageIcon thumbnail;

	 private JButton download;
	 private JButton view_selector;

	 private JCheckBox download_hidden;
	 private JCheckBox higher_directories;
	 private JCheckBox infinite;
	 private JCheckBox overwrite_existing;
	 private JCheckBox remove_failed;
	 private JCheckBox same_host;

	 private JScrollPane list_scroll;
	 private JScrollPane workspace_scroll;

	 private JTextField download_depth;
	 private JTextField number_downloads;
	 private JTextField number_each_host;
	 private JTextField source_url;

	 private JToggleButton destination_private;
	 private JToggleButton destination_shared;


	 /** Ensures that expansion and selection events between workspace trees based on the same model are synchronized. */
	 private TreeSynchronizer tree_sync = null;
	 /** A reference to the object that provides JNI ability. */
	 private WGet getter;
	 /** The default size of a label in the interface. */
	 static final private Dimension LABEL_SIZE = new Dimension(100,30);
	 /** The default size of the tree in the interface. */
	 static final private Dimension TREE_SIZE = new Dimension(250, 430);

	 public MirrorPane(TreeSynchronizer workspace_tree_sync) {
		  this.tree_sync = workspace_tree_sync;
		  getter = new WGet();
		  getter.start();
	 }

	 public void actionPerformed(ActionEvent event) {
		  Object event_source = event.getSource();
		  if(event_source == download) {
				// Check we have a valid url.
				GURL target_gurl = new GURL(source_url.getText());
				if(target_gurl.getURL() != null) {
					 // Update configuration settings.
					 Gatherer.config.set("mirroring.page_requisites", false, download_hidden.isSelected());
					 Gatherer.config.set("mirroring.no_parents", false, !higher_directories.isSelected());
					 Gatherer.config.set("mirroring.clobber", false, overwrite_existing.isSelected());
					 Gatherer.config.set("mirroring.remove_failed", false, remove_failed.isSelected());
					 Gatherer.config.set("mirroring.other_hosts", false, !same_host.isSelected());
					 // Depth
					 if(infinite.isSelected()) {
						  Gatherer.config.setInt("mirroring.depth", false, -1);
					 }
					 else if(download_depth.getText().length() > 0){
						  //Gatherer.println("Download_depth = " + download_depth.getText().trim());
						  Gatherer.config.setInt("mirroring.depth", false, Integer.parseInt(download_depth.getText().trim()));
					 }

					 // Queue download
					 if(destination_shared.isSelected()) {
						  getter.newJob(new GURL(source_url.getText()), model, Utility.CACHE_DIR);
					 }
					 else {
						  getter.newJob(new GURL(source_url.getText()), model, Gatherer.c_man.getCollectionCache());
					 }
				}
				// Otherwise indicate to the user that they've done something
				// wrong.
				else {
					 // Error message box.
					 String args[] = new String[1];
					 args[0] = source_url.getText();
					 message(Message.ERROR, get("Malformed_URL", args));
					 // Colour source_url red
					 source_url.setForeground(Color.red);
				}

		  }
		  else if(event_source == source_url) {
				// They've typed something. Change colour back to black
				source_url.setForeground(Color.black);
		  }
	 }

	 public void collectionChanged(boolean ready) {
		  // Some buttons are only enabled if a collection is open.
		  if(ready) {
				destination_private.setEnabled(true);
		  }
		  else {
				destination_shared.setSelected(true);
				destination_private.setEnabled(false);
		  }
		  // Retrieve appropriate workspace
		  model = (FileSystemModel) Gatherer.c_man.getWorkspace();
		  workspace_tree.setModel(model);
		  tree_sync.add(workspace_tree);
	 }

	 public void display() {
		  this.setLayout(new BorderLayout());

		  // Create
		  JPanel tree_pane = new JPanel(new BorderLayout());
		  tree_pane.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		  tree_pane.setMinimumSize(TREE_SIZE);
		  tree_pane.setSize(TREE_SIZE);
		  tree_pane.setPreferredSize(TREE_SIZE);

		  //Gatherer.println("\tCreating tree_label");
		  JLabel tree_label = new JLabel(get("Workspace_Tree"));

		  //Gatherer.println("\tCreating public tree");
		  model = (FileSystemModel) Gatherer.c_man.getWorkspace();
		  workspace_tree = new DragTree("Workspace_Mirror", model, null);
		  workspace_tree.getSelectionModel().setSelectionMode(TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);
		  workspace_tree.putClientProperty("JTree.lineStyle", "Angled");
		  workspace_tree.setBackgroundNonSelectionColor(Gatherer.config.getColor("coloring.workspace_tree_background", false));
		  workspace_tree.setTextNonSelectionColor(Gatherer.config.getColor("coloring.workspace_tree_foreground", false));
		  workspace_tree.setRootVisible(false);
		  workspace_tree.setBackgroundSelectionColor(Gatherer.config.getColor("coloring.workspace_selection_background", false));
		  workspace_tree.setTextSelectionColor(Gatherer.config.getColor("coloring.workspace_selection_foreground", false));

  		  // Add to a dummy drag-group.
  		  DragGroup group = new DragGroup();
  		  group.add(workspace_tree);
  		  group = null;

		  workspace_filter = Gatherer.g_man.getFilter(workspace_tree);
		  workspace_filter.setBackground(Gatherer.config.getColor("coloring.workspace_heading_background", false));
		  // Change the default colours of this filters combobox.
		  GComboBox fcb = workspace_filter.getComboBox();
		  fcb.setBackgroundNonSelectionColor(Gatherer.config.getColor("coloring.workspace_tree_background", false));
		  fcb.setTextNonSelectionColor(Gatherer.config.getColor("coloring.workspace_tree_foreground", false));
		  fcb.setBackgroundSelectionColor(Gatherer.config.getColor("coloring.workspace_selection_background", false));
		  fcb.setTextSelectionColor(Gatherer.config.getColor("coloring.workspace_selection_foreground", false));
		  fcb = null;

		  tree_pane.add(tree_label, BorderLayout.NORTH);
		  tree_pane.add(new JScrollPane(workspace_tree), BorderLayout.CENTER);
		  tree_pane.add(workspace_filter, BorderLayout.SOUTH);

		  // Create control_pane
		  JPanel control_pane = new JPanel(new BorderLayout());

		  //Gatherer.println("\tCreating control_label");
		  JLabel control_label = new JLabel(get("Download_Controls"));
		  control_label.setHorizontalAlignment(JLabel.CENTER);
		  control_label.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));

		  //central_pane = new JPanel(new GridLayout(7,1));
		  JPanel central_pane = new JPanel();
		  central_pane.setLayout(new BoxLayout(central_pane, BoxLayout.Y_AXIS));

		  //Gatherer.println("\tCreating source_url_pane");
		  JPanel source_url_pane = new JPanel(new BorderLayout());
		  JLabel source_url_label = new JLabel(get("Source_URL"));
		  source_url_label.setPreferredSize(LABEL_SIZE);
		  source_url = new JTextField("");
		  source_url_pane.add(source_url_label, BorderLayout.WEST);
		  source_url_pane.add(source_url, BorderLayout.CENTER);


		  //Gatherer.println("\tCreating download_depth");
		  JPanel download_depth_pane = new JPanel(new FlowLayout(FlowLayout.LEFT));
		  JLabel download_depth_label = new JLabel(get("Download_Depth"));
		  // Default depth. Its an int so we'll do some magic handwaving to magic it into a String. Alakazam.
		  download_depth = new JTextField(Utility.getDepthString(6));
		  infinite = new JCheckBox("Infinite");
		  download_depth_pane.add(download_depth_label);
		  download_depth_pane.add(download_depth);
		  download_depth_pane.add(infinite);

		  //Gatherer.println("\tCreating destination_folder");
		  JPanel destination_folder_pane = new JPanel(new FlowLayout(FlowLayout.LEFT));
		  JLabel destination_folder_label = new JLabel(get("Destination_Folder"));
		  destination_folder_options = new ButtonGroup();
		  destination_private = new JToggleButton(get("Destination_Private"));
		  destination_private.setEnabled(Gatherer.c_man != null && Gatherer.c_man.ready());
		  destination_private.setIcon(Utility.OFF_ICON);
		  destination_private.setSelectedIcon(Utility.ON_ICON);
		  destination_private.setSelected(false);
		  destination_private.addActionListener(this);
		  destination_folder_options.add(destination_private);
		  destination_shared = new JToggleButton(get("Destination_Shared"));
		  destination_shared.setIcon(Utility.OFF_ICON);
		  destination_shared.setSelectedIcon(Utility.ON_ICON);
		  destination_shared.setSelected(true);
		  destination_shared.addActionListener(this);
		  destination_folder_options.add(destination_shared);
		  destination_folder_pane.add(destination_folder_label);
		  destination_folder_pane.add(destination_private);
		  destination_folder_pane.add(destination_shared);

		  //Gatherer.println("\tCreating download_hidden");
		  download_hidden = new JCheckBox(get("Download_Hidden"));
		  // Why the friken heil is the horizontal alignment called Y?
		  download_hidden.setAlignmentY(Component.LEFT_ALIGNMENT);
		  download_hidden.setSelected(Gatherer.config.get("mirroring.page_requisites", false));

		  //Gatherer.println("\tCreating higher_directories");
		  higher_directories = new JCheckBox(get("Higher_Directories"));
		  higher_directories.setSelected(!Gatherer.config.get("mirroring.no_parents", false));

		  //Gatherer.println("\tCreating same_host");
		  same_host = new JCheckBox(get("Same_Host"));
		  same_host.setSelected(!Gatherer.config.get("mirroring.other_hosts", false));

		  //Gatherer.println("\tCreating overwrite");
		  overwrite_existing = new JCheckBox(get("Overwrite_Existing"));
		  overwrite_existing.setSelected(Gatherer.config.get("mirroring.overwrite", false));

		  JLabel further_options = new JLabel(get("Further_Options"));
		  further_options.setHorizontalAlignment(JLabel.LEFT);

		  central_pane.add(source_url_pane);
		  central_pane.add(download_depth_pane);
		  central_pane.add(destination_folder_pane);

		  JPanel three_mid = new JPanel(new GridLayout(5,1));

		  three_mid.add(download_hidden);
		  three_mid.add(higher_directories);
		  three_mid.add(same_host);
		  three_mid.add(overwrite_existing);
		  three_mid.add(further_options);

		  central_pane.add(three_mid);

		  //Gatherer.println("\tCreating thumbnail");
		  thumbnail = new ImageIcon(Utility.RES_DIR + "no_thumbnail.gif");
		  JLabel thumbnail_holder = new JLabel(thumbnail);
		  // Default thumbnail size.
		  //thumbnail_holder.setSize(Gatherer.config.getDimension("mirroring.thumbnail_size", false));
		  thumbnail_holder.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));

		  JPanel lower_pane = new JPanel(new BorderLayout());
		  lower_pane.setBorder(BorderFactory.createTitledBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED), get("Download_List_Configuration")));

		  remove_failed = new JCheckBox(get("Remove_Failed"));
		  remove_failed.setSelected(Gatherer.config.get("mirroring.remove_failed", false));

		  //Gatherer.println("\tCreating download_button");
		  download = new JButton(get("Download"));
		  download.addActionListener(this);
		  download.setHorizontalAlignment(JLabel.CENTER);
		  JPanel download_pane = new JPanel(new BorderLayout());

		  lower_pane.add(remove_failed, BorderLayout.NORTH);
		  lower_pane.add(download_pane, BorderLayout.CENTER);

		  control_pane.add(control_label, BorderLayout.NORTH);
		  control_pane.add(central_pane, BorderLayout.WEST);
		  //control_pane.add(thumbnail_holder, BorderLayout.CENTER);
		  control_pane.add(lower_pane, BorderLayout.SOUTH);

		  // Create list_pane
		  JPanel list_pane = new JPanel(new BorderLayout());
		  list_pane.setPreferredSize(new Dimension(Gatherer.g_man.getSize().width, Gatherer.g_man.getSize().height / 4));

		  download_pane.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		  download_pane.add(download);

		  //Gatherer.println("\tCreating list");
		  list_scroll = getter.getJobList();

		  //list_pane.add(download_pane, BorderLayout.NORTH);
		  list_pane.add(list_scroll, BorderLayout.CENTER);

		  // Add to content
		  JPanel temp_pane = new JPanel(new BorderLayout());
		  this.add(tree_pane, BorderLayout.WEST);
		  this.add(control_pane, BorderLayout.CENTER);
		  //temp_pane.add(control_pane, BorderLayout.NORTH);
		  //temp_pane.add(list_pane, BorderLayout.CENTER);
		  this.add(list_pane, BorderLayout.SOUTH);
	 }

	 public void setURL(String url) {
		  source_url.setText(url);
	 }

	 private String get(String key) {
		  return get(key, null);
	 }

	 private String get(String key, String args[]) {
		  if(key.indexOf('.') == -1) {
				key = "Mirroring." + key;
		  }
		  return Gatherer.dictionary.get(key,args);
	 }

	 /** Creates and dispatches a message given the initial details.
	  * @param level An int indicating the message level for this message.
	  * @param message A String which contains the payload of this message.
	  */
	 private void message(int level, String message) {
		  Message msg = new Message(Message.MIRRORING, level, message);
		  if(Gatherer.g_man != null) {
				Gatherer.g_man.message_pane.show(msg);
		  } else {
				Gatherer.println(msg.toString());
		  }
	 }
}
